from typing import Union
from dotenv import load_dotenv
import pyodbc # Python interface for ODBC API         https://github.com/mkleehammer/pyodbc
import os
from datetime import datetime
import logging
import platform

def get_read_connection(driver: Union[str, None]=None) -> pyodbc.Connection:
    """Function for getting a connection using the configured read environment.

    Allowed connection protocols include 'ActiveDirectoryIntegrated', 'ActiveDirectoryInteractive', 'ActiveDirectoryMsi' and 'SqlPassword'
    and is controlled by the 'IMBALANCE_LOG_PROTOCOL' environment parameter. Further the 'IMBALANCE_LOG_ASSIGNED_UID' variable is needed
    for user assigned identities

    :param str driver: Configurable driver override, defaults to system determinable

    :returns: Read connection to the imbalance server
    :rtype: pyodbc.Connection
    """
    try:
        load_dotenv(verbose=False, override=True)
    except:
        logging.info('Error in load_dotenv')

    # Extract the database properties
    server=os.environ['IMBALANCE_LOG_SERVER']
    database=os.environ['IMBALANCE_LOG_DATABASE']

    # Define the authentication protocol
    protocol=os.environ.get('IMBALANCE_LOG_PROTOCOL', 'SqlPassword')

    # Allow for driver override
    if driver is None:
        driver='ODBC Driver 17 for SQL Server'
        if platform.system() == 'Windows':
            driver='SQL Server'

    # Set the base connection string
    connection_string = 'DRIVER={' + driver + '};SERVER='+server+';DATABASE='+database

    # Update connection string for username and password if needed
    if protocol not in ['ActiveDirectoryIntegrated', 'ActiveDirectoryInteractive', 'ActiveDirectoryMsi']:
        username=os.environ['IMBALANCE_LOG_READ_USERNAME']
        password=os.environ['IMBALANCE_LOG_READ_PASSWORD']
        connection_string += ';UID=' + username + ';PWD=' + password
    else:
        connection_string += ';Authentication=' + protocol

        # Assign the UID for user assigned identities
        if protocol == 'ActiveDirectoryMsi':
            user_id = os.environ.get('IMBALANCE_LOG_ASSIGNED_UID', None)
            if user_id is not None:
                connection_string += ';UID=' + user_id


    tries=0
    while tries < 5:
        try:
            connection = pyodbc.connect(connection_string)
            break
        except:
            tries = tries + 1
            logging.info('get_connection retry')
    
    return connection




# Install deps with: pip install requests
import os
import requests

from meimbalance import sqllog

def send_sms(recipient: int, msg_content: str, sender: str='AE KF'):
	if len(sender) > 11:
		error_message = 'Sender string too long: ' + sender
		sqllog.log_application('ERROR', error_message, 'meimbalance.sms')
		raise Exception(error_message)

	token = ""
	try:
		token=os.environ['GATEWAYAPI_TOKEN']
	except Exception as e:
		sqllog.log_application('ERROR', str(e), 'meimbalance.sms')
		raise

	payload = {
		"sender": sender,
		"message": msg_content,
		"recipients": [
			{"msisdn": recipient}
		],
	}
	response = ""
	try:
		resp = requests.post(
			"https://gatewayapi.com/rest/mtsms",
			json=payload,
			auth=(token, ""),
		)
		resp.raise_for_status()
		response=resp.json()
		sqllog.log_application('INFO', str(payload), 'meimbalance.sms')
		sqllog.log_application('INFO', str(response), 'meimbalance.sms')

	except requests.exceptions.HTTPError as e:
		sqllog.log_application('ERROR', str(e), 'meimbalance.sms')
		raise
	
	return response



from typing import Union
from dotenv import load_dotenv
import pyodbc # Python interface for ODBC API         https://github.com/mkleehammer/pyodbc
import os
from datetime import datetime
from datetime import timezone
import logging
import platform
import time

def get_connection(driver: Union[str, None]=None) -> pyodbc.Connection:
	"""Function for getting a connection using the configured environment.

	Allowed connection protocols include 'ActiveDirectoryIntegrated', 'ActiveDirectoryInteractive', 'ActiveDirectoryMsi' and 'SqlPassword'
	and is controlled by the 'IMBALANCE_LOG_PROTOCOL' environment parameter

	:param str driver: Configurable driver override, defaults to system determinable

	:returns: Read connection to the imbalance server
	:rtype: pyodbc.Connection
	"""
	try:
		load_dotenv(verbose=False, override=True)
	except:
		logging.info('Error in load_dotenv')

	server=os.environ['IMBALANCE_LOG_SERVER']
	database=os.environ['IMBALANCE_LOG_DATABASE']

	# Define the authentication protocol
	protocol=os.environ.get('IMBALANCE_LOG_PROTOCOL', 'SqlPassword')

	# Allow for driver override
	if driver is None:
		driver='ODBC Driver 17 for SQL Server'
		if platform.system() == 'Windows':
			driver='SQL Server'

	# Set the base connection string
	connection_string = 'DRIVER={' + driver + '};SERVER='+server+';DATABASE='+database

	# Update connection string for username and password if needed
	if protocol not in ['ActiveDirectoryIntegrated', 'ActiveDirectoryInteractive', 'ActiveDirectoryMsi']:
		username=os.environ['IMBALANCE_LOG_USERNAME']
		password=os.environ['IMBALANCE_LOG_PASSWORD']
		connection_string += ';UID=' + username + ';PWD=' + password
	else:
		connection_string += ';Authentication=' + protocol

		# Assign the UID for user assigned identities
		if protocol == 'ActiveDirectoryMsi':
			user_id = os.environ.get('IMBALANCE_LOG_ASSIGNED_UID', None)
			if user_id is not None:
				connection_string += ';UID=' + user_id

	# Connect to the database
	tries=0
	connect=False
	while tries < 5:
		try:
			connection = pyodbc.connect(connection_string)
			connect = True
			break
		except:
			tries = tries + 1
			time.sleep(0.1)
			logging.info('get_connection retry')
	# If we are not connected after 5 tries, retry once more and let the exception abort the operation
	if connect == False:
		connection = pyodbc.connect(connection_string)

	return connection

def log_files(filetype, filename, url, status, message):
    now = datetime.now(tz=timezone.utc)
    connection = get_connection()
    cursor = connection.cursor()
    cursor.execute('insert into files(dt, filetype, filename, url, status, message) values(?, ?, ?, ?, ?, ?)', now, filetype, filename, url, status, message)
    connection.commit()

def log_files_forecast(filetype, filename, url, status, message,starttime,filecount=0,elapsedtime=0,year='0000',month='00',day='00',forecast='0000',windpark=''):
    message = message[0:4000]
    now = datetime.now(tz=timezone.utc)
    connection = get_connection()
    cursor = connection.cursor()
    cursor.execute('insert into files(dt, filetype, filename, url, status, message, forecast,filecount,elapsedtime,year,month,day,starttime,windpark) values(?, ?, ?, ?, ?, ?, ?,?,?,?,?,?,?,?)', now, filetype, filename, url, status, message, forecast,filecount,elapsedtime,year,month,day,starttime,windpark)
    connection.commit()

def log(severity, message):
    now = datetime.now(tz=timezone.utc)
    connection = get_connection()
    cursor = connection.cursor()
    cursor.execute('insert into logs(dt, severity, message) values(?, ?, ?)', now, severity, message)
    connection.commit()

def log_application(severity, message, application):
    message = message[0:4000]
    now = datetime.now(tz=timezone.utc)
    connection = get_connection()
    cursor = connection.cursor()
    cursor.execute('insert into logs(dt, severity, message, application) values(?, ?, ?, ?)', now, severity, message, application)
    connection.commit()

def log_application_park(severity, message, application, windpark):
    message = message[0:4000]
    now = datetime.now(tz=timezone.utc)
    connection = get_connection()
    cursor = connection.cursor()
    cursor.execute('insert into logs(dt, severity, message, application, park) values(?, ?, ?, ?, ?)', now, severity, message, application, windpark)
    connection.commit()
# 
# Query the log database for a single row single column
#
def __get_sql_value(sql):
    connection = get_connection()
    cursor = connection.cursor()
    cursor.execute(sql)
    for row in cursor.fetchall():
        value = row.value
        break

    cursor.close()
    return value 




# using SendGrid's Python Library
# https://github.com/sendgrid/sendgrid-python
import os
from sendgrid import SendGridAPIClient
from sendgrid.helpers.mail import Mail
from meimbalance import sqllog 

def send_mail(recipient: str, subject: str, msg_content: str, sender: str='noreply-kraftforvaltning@ae.no'):
	message = Mail(
		from_email=sender,
		to_emails=recipient,
		subject=subject,
		html_content=msg_content)
	try:
		sg = SendGridAPIClient(os.environ.get('SENDGRID_API_KEY'))
		response = sg.send(message)
		sqllog.log_application('INFO', 'Mail with subject ' + subject + ' sent to ' + recipient, 'meimbalance.mail')
		sqllog.log_application('DEBUG', str(response.headers), 'meimbalance.mail')
	except Exception as e:
		sqllog.log_application('ERROR', str(e.message), 'meimbalance.mail')